<!--
#TITLE Free Pascal - How To CVS
#ENTRY develop
#SUBENTRY How to CVS
#HEADGIF Develop
#HEADALT Development
-->

<H3><a name="cvs">Connecting with GNU CVS</a></H3>

<ol>
<!-- IDXSTART -->
  <LI><A HREF="#introduc">CVS, an introduction</A></LI>
  <LI><A HREF="#obtaincvs">Obtaining CVS</A></LI>
  <LI><A HREF="#setup">Installing, and setting up CVS</A></LI>
  <LI><A HREF="#login">Logging in to the CVS server</A></LI>
  <LI><A HREF="#checkout">Initial download (checkout) of a package</A></LI>
  <LI><A HREF="#update">Updating a package</A></LI>
<!-- IDXEND -->
</ol>
This document was last updated: januari 18th 2000. Comments and suggestions to
<a href="mailto:marco@freepascal.org">Marco@freepascal.org</a>.
<ol>
<p>

</p>
<A NAME="introduc"></A>
<H3><LI>CVS, an introduction</LI></H3>
<p>
<A HREF="http://www.cyclic.com/">CVS</A> is a <a href="http://www.gnu.org">GNU</a> application which simplifies
version management and working on big projects with several developers at
once.
</p>

<p>
As an alternative to the daily zip files of the CVS sources,
the FreePascal CVS repository has been made accessible for everyone,
with read-only access. This means that the entire project's
code can be accessed at any minute.
</p>

<p>
The minute that a developer commits a fix to the repository, it is available to
everyone. CVS, when used properly, is also the most economic form of obtaining the sources,
specially if you want to update the sources at a rate of once every two weeks
or more often. After the initial big download, the daily updates are quite small
and fast.
</p>

<A NAME="obtaincvs"></A>
<H3><LI>Obtaining CVS</LI></H3>
<p>
<ol>
<lI><b>Linux/FreeBSD</b>, Most major Linux distributions include CVS
 The distribution's packagemanager can be used to install CVS. 
(Slackware pkgtool, RedHat: RPM or its X-windows companion). CVS uses RCS, so
RCS should also be installed. Both packages together are about 5 MB diskspace 
depending on the distribution.</li>
<lI><b>Windows</b>, The easiest way is to download WinCVS (about 1.7 MB)
available on <a href="www.wincvs.org">www.wincvs.org</a>. This archive 
also contains a cvs.hlp which is a nice reference about CVS.</li>
<li><b>MAC</b>, A client is available on the <a href="http://www.wincvs.org">WinCVS site</a>
<li><b>Other OSes</b>, Probably a suitable client exists on the
<A HREF="http://www.cyclic.com/">CVS web site</a></li>
</ol>

<A NAME="setup"></A>
<H3><LI>Installing, and setting up CVS</LI></H3>

First create an empty base directory for the sources: e.g.<p>
<pre>
mkdir /usr/src/fpc/     on linux  or
md    d:\source\fpc\    on dos,
</pre>
the location is free, but you must have write access for that directory.<p>

This directory is the base source directory, and all cvs commands described
below are executed in this directory.<p>

The WINCVS directory must also be added to the PATH. The following example
works for 4Dos and DOS:<p>

<pre>
path %PATH%;c:\tools\wincvs
</pre>
<p>
You can also add the directory to the path line in autoexec.bat
</P>
Also CVS uses a directory for some very small configuration files. Under
Unix/Linux the home directory is automatically used for this purpose, but 
under DOS/Win32 the directory to be used for this purpose has to be
specified, by setting the HOME environment variable:
</p>

<pre>
set HOME=c:\tools\wincvs
</pre>
<p>
Any directory NOT under cvs control can be used. So don't specify the
directory of the repository as your HOME directory.
</P>
<p>
To avoid unnecessary long commandlines we also specify the server to which
CVS should connect in a environment variable on both Linux and Dos:
</P>


Dos: (in your autoexec.bat or login batchfile)
<pre>
 set CVSROOT=:pserver:cvs@cvs.freepascal.org:/FPC/CVS
</pre>

Linux(bash): (put in your .bash_profile)

<pre>
 export CVSROOT=":pserver:cvs@cvs.freepascal.org:/FPC/CVS"
</pre>

<A NAME="login"></A>
<H3><LI>Logging in to the CVS server</LI></H3>
<p>
You only have to log in using this procedure once. Clients will store the
login information in a .cvspass file in your HOME directory.<br>
</p>

The login command is

<pre>
cvs login
</pre>

or if you didn't set the CVSROOT environment variable:

<pre>
cvs -d :pserver:cvs@cvs.freepascal.org:/FPC/CVS login
</pre>

<p>
The client will prompt for a password:

<PRE>
(Logging in to cvs@cvs.freepascal.org)
CVS password:
</PRE>

which is &quot;cvs&quot; without quotes and in lower case.
</p>

Under Win32 the following batchfile will ease this proces:<p>

<pre>
@echo off
path %PATH%;d:\tools\wincvs
set home=d:\cvssource\fpc
set CVSROOT=:pserver:cvs@cvs.freepascal.org:/FPC/CVS
echo enter password (="cvs")
cvs login
</pre>
<p>
<A NAME="checkout"></A>
<H3><LI>Initial download (checkout) of a package</LI></H3>
<p>
To perform the initial download(called checkout) of a package:
</P>

<pre>
cvs -z 3 checkout &lt;modulename&gt;
</pre>

or (if you didn't set the CVSROOT environment variable)

<pre>
cvs -z 3 -d :pserver:cvs@cvs.freepascal.org:/FPC/CVS checkout  &lt;modulename&gt;
</pre>

where &lt;modulename&gt; is one of the following modules:

<UL>
<LI><B>base</B>, contains some makefile templates, not necessary for normal compilation</li>
<LI><B>rtl</B>, contains the RTL (Run-Time Library) source.</li>
<LI><B>compiler</B> contains the compiler source (needs RTL to build)</lI>
<LI><B>fcl</B> contains the Free Component Library source. (depends on PACKAGES/paszlib)</li>
<li><b>utils</b> contains the source of some utils used inside the FPC project (some need FCL)</li>
<LI><B>gtk</B> contains the GTK bindings source.</li>
<LI><B>api</B> contains the API source (needed by FV, and everything based on it).</li>
<LI><B>docs</B> contains the documentation (TeX) source.</LI>
<li><b>gdbpas</b> contains a patch to GDB sources to support pascal in GDB. You'll need gcc (or djgpp)
            and preferably the v4.18 GDB sources to compile.</li>
<li><b>ide</b> contains the textmode IDE. This doesn't compile at the moment since the free vision
source is not publicly available due to copyright problems</li>
<LI><B>bugs</B> contains the Bugs repository.
<li><b>install</b> Contains files added to, and used to create releases, (RPM spec files, the installer source, and the demos)</li>
<!--
<Li><b>kcl</b> The Kassandra/KCL project which aims to create a portable visual component library not necessarily resembling Delphi's (needs FCL and GTK)</lI>
-->
<li><b>lazarus</b> The Lazarus project which aims to create a Delphi compatible VCL</lI>
<li><b>packages</b> Several smaller packages and the KCL/Kassandra project (
api, forms, gdbint, ggi, gtk, ibase, inet, mysql, ncurses,
opengl, paszlib, postgres, svgalib, uncgi, utmp, x11, zlib)</li>
</ul>

<p>
Checking out the entire CVS archive takes about 40 minutes with a 44k0
connection. (compare: downloading only rtl and compiler source zips once 
takes about 20 minutes at the same speed) <p>
</p>

<A NAME="update"></A>
<H3><LI>Updating a package</LI></H3>
<p>
The advantage; updating a checked out package the next day will be quick.
Typically, about 1 1/2 minute is needed
to update the entire project (assuming the repository was also updated the day
before, and the speed of the link is 44Kbaud)
</P>

The syntax for the update command is similar to the checkout command:<p>

<pre>
cvs -z 3 update -Pd &lt;modulename&gt;
</pre>

or for the case when  CVSROOT isn't set:<p> 

<pre>
cvs -z 3 -d :pserver:cvs@cvs.freepascal.org:/FPC/CVS update -Pd &lt;modulename&gt;
</pre>

<p>
The -Pd switch tells your client to remove empty directories (which should never
be deleted by hand!). Also don't move the -Pd switch before the update command, or any of the
other switches after the update command. The order of the switches is important!
</P>
The -z 3 switch tells the CVS server to compress all data before sending. Don't
use maximal compression (-z 9), since it is hardly faster, and put huge loads on the
server.<p>

The following batchfile works with the Win32 CVS(wincvs) client to update the entire repository: (if you have CVSROOT set,
otherwise add the -d parameter to each cvs line)<p>

<pre>
@echo off
cvs  -z 3 update -Pd compiler
cvs  -z 3 update -Pd rtl
cvs  -z 3 update -Pd api
cvs  -z 3 update -Pd fcl
cvs  -z 3 update -Pd ide
cvs  -z 3 update -Pd utils
cvs  -z 3 update -Pd tests
cvs  -z 3 update -Pd packages
cvs  -z 3 update -Pd bugs
cvs  -z 3 update -Pd docs
cvs  -z 3 update -Pd gdbpas
cvs  -z 3 update -Pd gtk
cvs  -z 3 update -Pd install
cvs  -z 3 update -Pd kcl
cvs  -z 3 update -Pd lazarus
cvs  -z 3 update -Pd base
cvs  -z 3 update -Pd html
</pre>

An equivalent for Linux would look like this (again assuming CVSROOT is
set):<p>

<pre>
#/bin/sh
cvs  -z 3 update -Pd compiler 
cvs  -z 3 update -Pd rtl 
cvs  -z 3 update -Pd api 
cvs  -z 3 update -Pd fcl 
cvs  -z 3 update -Pd ide 
cvs  -z 3 update -Pd utils 
cvs  -z 3 update -Pd tests 
cvs  -z 3 update -Pd packages 
cvs  -z 3 update -Pd bugs 
cvs  -z 3 update -Pd docs 
cvs  -z 3 update -Pd gdbpas 
cvs  -z 3 update -Pd gtk 
cvs  -z 3 update -Pd install 
cvs  -z 3 update -Pd kcl 
cvs  -z 3 update -Pd lazarus 
cvs  -z 3 update -Pd base 
cvs  -z 3 update -Pd html 
</pre>

<P>
For the curious and experimental: You have only read-only access, so don't try to commit
anything :-)</p>
</HTML>
